package com.salpa.subject.mapper;

import com.salpa.subject.domain.MonitorDataSource;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;

/**
 * @author zhuff
 */
@Mapper
public interface DatabaseMapper {

    void addDatabaseSource(MonitorDataSource monitorDataSource);

    MonitorDataSource selectDataSourceById(Integer id);

    void updateDateSource(MonitorDataSource monitorDataSource);

    void deleteDataSource(Integer projectId);

    List<MonitorDataSource> selectDataSourceList();

    MonitorDataSource selectByDatabaseName(String databaseName);
}
